/* This is part of Calcyte, a tool for implementing the DataCrate data packaging
spec.  Copyright (C) 2018  University of Technology Sydney

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Bags a datacrate wich already has a metadata.json file */

const path = require("path");
const shell = require("shelljs");
const tmp = require("tmp");
const fs = require("fs");
const defaults = require("./defaults.js");

module.exports = function() {
  return {
    targetDir: this.targetDir,
    metadata_path: this.metadata_path,

    generate_bag_info: function generate_bag_info() {
      this.bag_meta = {
        "ROCrate_Specification_Identifier":
          this.crate.defaults.ROCrate_Specification_Identifier
      };


      if (this.root["contactPoint"] && this.root["contactPoint"]["@id"]) {
        contact = this.crate.getItem(this.root["contactPoint"]["@id"]);
        map = {
          email: "Contact-Email",
          phone: "Contact-Telephone",
          name: "Contact-Name"
        };
        
        for (var [k, v] of Object.entries(map)) {
          if (contact[k]) {
            this.bag_meta[v] = String(contact[k]);
          }
        }
      }

      if (this.root["publisher"] && this.root["publisher"]["@id"]) {
        publisher = this.crate.getItem(this.root["publisher"]["@id"]);
        
        if(publisher["name"]) {
          this.bag_meta["SourceOrganization"] = publisher.name
          
        }
      }
      if (this.root["description"]) {
        this.bag_meta["External-Description"] = this.root["description"];
      }
      this.bag_meta["Bagging-Date"] = new Date().toISOString();
      // Return a hash of BagIt style metadata by looking for it in the JSON-LD structure


    },
   

    save_bag_info: function save_bag_info() {
      var bag_info = "";
      for (var [k, v] of Object.entries(this.bag_meta)) {
        bag_info += k + ": " + v + "\n";
      }
      fs.writeFileSync(path.join(this.targetDir, "bag-info.txt"), bag_info);
    },
    update: function update_bag_tags() {
      shell.exec("bagit updatetagmanifests " + this.targetDir);
    },

   
    bag: function bag(source_dir, bag_dir, crate) {
      // TODO Generate a list of all files
      // FOR NOW: delete metadata.json and index.html
      // Generate bag info later

      this.crate = crate;
      this.crate.index(); 
      this.root = crate.getRootDataset();
      this.helper = crate.utils;
      this.targetDir = bag_dir;
  
      shell.exec(
        "bagit create --excludebaginfo " +
          this.targetDir +
          " " +
          path.join(source_dir, "*")
      );
     
      return this.targetDir;
    }
  };
};
