/* This is part of Calcyte a tool for implementing the DataCrate data packaging
spec.  Copyright (C) 2018  University of Technology Sydney

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

const defaults = require("./defaults");
const Preview = require("ro-crate").Preview
var ejs = require("ejs");
const fs = require("fs-extra");



module.exports = function () {
  return {
   
    init: function init(crate, template_path) {
      if (template_path) {
        var temp = fs.readFileSync(template_path, { encoding: "utf8" });
        this.template = ejs.compile(temp);
      }
      this.crate = crate;
      this.crate.index();
      //this.helper.add_back_links();
      // A container for our page
    },
   
    make_index_html: async function make_index_html(zip_path, render_script) {
      var body_el = "";
      var rootNode = this.crate.getRootDataset();
      var name = rootNode.name;
      if (!render_script) {
        render_script = defaults.render_script;
      }
      if (zip_path) {
        zip_path = `<a href='${zip_path}'>Download this Dataset</a>`;
           }
      const preview = new Preview(this.crate);
      const summary = await preview.summarizeDataset();
      var date = new Date(); 
      var timestamp = date.getTime(); 
      return this.template({
        html:summary.html(),
        dataset_name: name,
        item_name: name,
        citation: this.text_citation,
        zip_link: zip_path,
        up_link: "",
        time_stamp: timestamp,
        ROCrate_version: defaults.ROCrate_version,
        spec_id: defaults.DataCrate_Specification_Identifier,
        json_ld: JSON.stringify(this.crate.getJson(), null, 2),
        render_script: render_script
      })  
    }
    }
  }
